@@ -14,12 +14,15 @@ import requests_cache
 import logging
 import pandas as pd
 import numpy as np
-from datetime import datetime  # TODO use utils.to_datetime and add tests!
 
-from fastf1.utils import recursive_dict_get, to_timedelta
+from fastf1.utils import recursive_dict_get, to_timedelta, to_datetime
 
 base_url = 'https://livetiming.formula1.com'
 
+requests_session = None
+"""Can be used to create a custom requests session which is then used by
+all api calls"""
+
 headers = {
   'Host': 'livetiming.formula1.com',
   'Connection': 'close',
@@ -232,15 +235,19 @@ def timing_data(path, response=None, livedata=None):
       linear data stream into a usable object and because of frequent errors and inaccuracies in said stream.
       Occasionally an "educated guess" needs to be made for judging whether a value belongs to this lap or to another
       lap. Additionally, some values which are considered "obviously" wrong are removed from the data. This can happen
-      with or without warnings, depending on the reason an severity.
-      | Timestamps marking start or end of a lap are postprocessed as the provided values are inaccurate.
-      | Lap and sector times are not modified ever! They are considered as the absolute truth. If necessary,
-      other values are adjusted to fit.
+      with or without warnings, depending on the reason and severity.
+
+      - Timestamps ('SessionTime') marking start or end of a lap are
+        post-processed as the provided values are inaccurate.
+      - Lap and sector times are not modified ever! They are considered as the
+        absolute truth. If necessary, other values are adjusted to fit.
 
 
     Args:
         path: api path base string (see :func:`make_path`)
         response (optional): api response can be passed if data was already downloaded
+        livedata: An instance of :class:`fastf1.livetiming.data.LivetimingData`
+            to use as a source instead of the api
 
     Returns:
         (DataFrame, DataFrame):
@@ -293,7 +300,7 @@ def timing_data(path, response=None, livedata=None):
     # split up response per driver for easier iteration and processing later
     resp_per_driver = dict()
     for entry in response:
-        if 'Lines' not in entry[1]:
+        if (len(entry) < 2) or 'Lines' not in entry[1]:
             continue
         for drv in entry[1]['Lines']:
             if drv not in resp_per_driver.keys():
@@ -586,7 +593,7 @@ def _stream_data_driver(driver_raw, empty_vals, drv):
     for time, resp in driver_raw:
         new_entry = False
         if val := recursive_dict_get(resp, 'Position'):
-            drv_data['Position'][i] = val
+            drv_data['Position'][i] = int(val)
             new_entry = True
         if val := recursive_dict_get(resp, 'GapToLeader'):
             drv_data['GapToLeader'][i] = val
@@ -597,7 +604,7 @@ def _stream_data_driver(driver_raw, empty_vals, drv):
 
         # at least one value was present, create next row
         if new_entry:
-            drv_data['Time'][i] = time
+            drv_data['Time'][i] = to_timedelta(time)
             drv_data['Driver'][i] = drv
             i += 1
 
@@ -659,8 +666,10 @@ def timing_app_data(path, response=None, livedata=None):
             'TyresNotChanged': [], 'Time': [], 'LapFlags': [], 'LapCountTime': [], 'StartLaps': [], 'Outlap': []}
 
     for entry in response:
-        time = to_timedelta(entry[0])
+        if (len(entry) < 2) or 'Lines' not in entry[1]:
+            continue
 
+        time = to_timedelta(entry[0])
         row = entry[1]
         for driver_number in row['Lines']:
             if update := recursive_dict_get(row, 'Lines', driver_number, 'Stints'):
@@ -670,7 +679,12 @@ def timing_app_data(path, response=None, livedata=None):
                         stint = update[stint]
                     for key in data:
                         if key in stint:
-                            data[key].append(stint[key])
+                            val = stint[key]
+                            if key == 'LapTime':
+                                val = to_timedelta(val)
+                            elif key == 'New':
+                                val = True if val == 'true' else False
+                            data[key].append(val)
                         else:
                             data[key].append(None)
                     for key in stint:
@@ -689,14 +703,15 @@ def car_data(path, response=None, livedata=None):
     """Fetch and parse car data.
 
     Car data provides the following data channels per sample:
-        - Date (pandas.Timestamp): timestamp for this sample as Date + Time; more or less exact
         - Time (pandas.Timedelta): session timestamp (time only); inaccurate, has duplicate values; use Date instead
+        - Date (pandas.Timestamp): timestamp for this sample as Date + Time; more or less exact
         - Speed (int): Km/h
         - RPM (int)
-        - Gear (int) [called 'nGear' in the data!]
+        - Gear (int): [called 'nGear' in the data!]
         - Throttle (int): 0-100%
         - Brake (int): 0-100% (don't trust brake too much)
         - DRS (int): 0=Off, 8=Active; sometimes other values --> to be researched
+        - Source (str): Indicates the source of a sample; 'car' for all values here
 
     The data stream has a sample rate of (usually) 240ms. The samples from the data streams for position data and
     car data do not line up. Resampling/interpolation is required to merge them.
@@ -729,34 +744,47 @@ def car_data(path, response=None, livedata=None):
     logging.info("Parsing car data...")
 
     channels = {'0': 'RPM', '2': 'Speed', '3': 'nGear', '4': 'Throttle', '5': 'Brake', '45': 'DRS'}
-    columns = {'Time', 'Date', 'RPM', 'Speed', 'nGear', 'Throttle', 'Brake', 'DRS'}
+    columns = ['Time', 'Date', 'RPM', 'Speed', 'nGear', 'Throttle', 'Brake', 'DRS', 'Source']
+    ts_length = 12  # length of timestamp: len('00:00:00:000')
 
     data = dict()
+    decode_error_count = 0
 
     for record in response:
-        time = to_timedelta(record[0])
-        if is_livedata:
-            jrecord = parse(record[1], zipped=True)
-        else:
-            jrecord = record[1]
+        try:
+            if is_livedata:
+                time = to_timedelta(record[0])
+                jrecord = parse(record[1], zipped=True)
+            else:
+                time = to_timedelta(record[:ts_length])
+                jrecord = parse(record[ts_length:], zipped=True)
 
-        for entry in jrecord['Entries']:
-            # date format is '2020-08-08T09:45:03.0619797Z' with a varying number of millisecond decimal points
-            # always remove last char ('z'), max len 26, right pad to len 26 with zeroes if shorter
-            date = datetime.fromisoformat('{:<026}'.format(entry['Utc'][:-1][:26]))
+            for entry in jrecord['Entries']:
+                # date format is '2020-08-08T09:45:03.0619797Z' with a varying number of millisecond decimal points
+                # always remove last char ('z'), max len 26, right pad to len 26 with zeroes if shorter
+                date = to_datetime(entry['Utc'])
 
-            for driver in entry['Cars']:
-                if driver not in data:
-                    data[driver] = {col: list() for col in columns}
+                for driver in entry['Cars']:
+                    if driver not in data:
+                        data[driver] = {col: list() for col in columns}
 
-                data[driver]['Time'].append(time)
-                data[driver]['Date'].append(date)
+                    data[driver]['Time'].append(time)
+                    data[driver]['Date'].append(date)
 
-                for n in channels:
-                    val = recursive_dict_get(entry, 'Cars', driver, 'Channels', n)
-                    if not val:
-                        val = 0
-                    data[driver][channels[n]].append(val)
+                    for n in channels:
+                        val = recursive_dict_get(entry, 'Cars', driver, 'Channels', n)
+                        if not val:
+                            val = 0
+                        data[driver][channels[n]].append(int(val))
+
+        except Exception:
+            # too risky to specify an exception: unexpected invalid data!
+            decode_error_count += 1
+            continue
+
+    if decode_error_count > 0:
+        logging.warning(f"Car data: failed to decode {decode_error_count} "
+                        f"messages ({len(response)} messages total)")
 
     # create one dataframe per driver and check for the longest dataframe
     most_complete_ref = None
@@ -771,12 +799,14 @@ def car_data(path, response=None, livedata=None):
     for driver in data:
         if len(data[driver]['Date']) < len(most_complete_ref):
             # there is missing data for this driver
-            # extend the Date column and fill up missing telemetry values with zero,
-            # except Time which is left as NaT and will be calculated correctly during resampling
+            # extend the Date column and fill up missing telemetry values with
+            # zero, except Time which is left as NaT and will be calculated
+            # correctly based on Session.t0_date anyways when creating Telemetry
+            # instances in Session.load_telemetry
             index_df = pd.DataFrame(data={'Date': most_complete_ref})
             data[driver] = data[driver].merge(index_df, how='outer').sort_values(by='Date').reset_index(drop=True)
             data[driver].loc[:, channels.values()] = \
-                data[driver].loc[:, channels.values()].fillna(value=0, inplace=False)
+                data[driver].loc[:, channels.values()].fillna(value=0, inplace=False).astype('int64')
 
             logging.warning(f"Driver {driver: >2}: Car data is incomplete!")
 
@@ -788,10 +818,11 @@ def position_data(path, response=None, livedata=None):
     """Fetch and parse position data.
 
     Position data provides the following data channels per sample:
-        - Date (pandas.Timestamp): timestamp for this sample as Date + Time; more or less exact
         - Time (pandas.Timedelta): session timestamp (time only); inaccurate, has duplicate values; use Date instead
-        - X, Y, Z (int): Position coordinates; starting from 2020 the coordinates are given in 1/10 meter
+        - Date (pandas.Timestamp): timestamp for this sample as Date + Time; more or less exact
         - Status (str): 'OnTrack' or 'OffTrack'
+        - X, Y, Z (int): Position coordinates; starting from 2020 the coordinates are given in 1/10 meter
+        - Source (str): Indicates the source of a sample; 'pos' for all values here
 
     The data stream has a sample rate of (usually) 220ms. The samples from the data streams for position data and
     car data do not line up. Resampling/interpolation is required to merge them.
@@ -827,38 +858,49 @@ def position_data(path, response=None, livedata=None):
         return {}
 
     ts_length = 12  # length of timestamp: len('00:00:00:000')
-    columns = ['Time', 'Date', 'Status', 'X', 'Y', 'Z']
+    columns = ['Time', 'Date', 'Status', 'X', 'Y', 'Z', 'Source']
 
     data = dict()
+    decode_error_count = 0
 
     for record in response:
-        if is_livedata:
-            time = record[0]
-            jrecord = parse(record[1], zipped=True)
-        else:
-            time = to_timedelta(record[:ts_length])
-            jrecord = parse(record[ts_length:], zipped=True)
+        try:
+            if is_livedata:
+                time = record[0]
+                jrecord = parse(record[1], zipped=True)
+            else:
+                time = to_timedelta(record[:ts_length])
+                jrecord = parse(record[ts_length:], zipped=True)
 
-        for sample in jrecord['Position']:
-            # date format is '2020-08-08T09:45:03.0619797Z' with a varying number of millisecond decimal points
-            # always remove last char ('z'), max len 26, right pad to len 26 with zeroes if shorter
-            date = datetime.fromisoformat('{:<026}'.format(sample['Timestamp'][:-1][:26]))
+            for sample in jrecord['Position']:
+                # date format is '2020-08-08T09:45:03.0619797Z' with a varying number of millisecond decimal points
+                # always remove last char ('z'), max len 26, right pad to len 26 with zeroes if shorter
+                date = to_datetime(sample['Timestamp'])
 
-            for driver in sample['Entries']:
-                if driver not in data:
-                    data[driver] = {col: list() for col in columns}
+                for driver in sample['Entries']:
+                    if driver not in data:
+                        data[driver] = {col: list() for col in columns}
 
-                data[driver]['Time'].append(time)
-                data[driver]['Date'].append(date)
+                    data[driver]['Time'].append(time)
+                    data[driver]['Date'].append(date)
 
-                for coord in ['X', 'Y', 'Z']:
-                    data[driver][coord].append(recursive_dict_get(sample, 'Entries', driver, coord))
+                    for coord in ['X', 'Y', 'Z']:
+                        data[driver][coord].append(recursive_dict_get(sample, 'Entries', driver, coord))
 
-                status = recursive_dict_get(sample, 'Entries', driver, 'Status')
-                if str(status).isdigit():
-                    # Fallback on older api status mapping and convert
-                    status = 'OffTrack' if int(status) else 'OnTrack'
-                data[driver]['Status'].append(status)
+                    status = recursive_dict_get(sample, 'Entries', driver, 'Status')
+                    if str(status).isdigit():
+                        # Fallback on older api status mapping and convert
+                        status = 'OffTrack' if int(status) else 'OnTrack'
+                    data[driver]['Status'].append(status)
+
+        except Exception:
+            # too risky to specify an exception: unexpected invalid data!
+            decode_error_count += 1
+            continue
+
+    if decode_error_count > 0:
+        logging.warning(f"Position data: failed to decode {decode_error_count} "
+                        f"messages ({len(response)} messages total)")
 
     # create one dataframe per driver and check for the longest dataframe
     most_complete_ref = None
@@ -873,8 +915,10 @@ def position_data(path, response=None, livedata=None):
     for driver in data:
         if len(data[driver]['Date']) < len(most_complete_ref):
             # there is missing data for this driver
-            # extend the Date column and fill up missing telemetry values with zero,
-            # except Time which is left as NaT and will be calculated correctly during resampling
+            # extend the Date column and fill up missing telemetry values with
+            # zero, except Time which is left as NaT and will be calculated
+            # correctly based on Session.t0_date anyways when creating Telemetry
+            # instances in Session.load_telemetry
             # and except Status which should be 'OffTrack' for missing data
             index_df = pd.DataFrame(data={'Date': most_complete_ref})
             data[driver] = data[driver].merge(index_df, how='outer').sort_values(by='Date').reset_index()
@@ -893,7 +937,7 @@ def track_status_data(path, response=None, livedata=None):
     Track status contains information on yellow/red/green flags, safety car and virtual safety car. It provides the
     following data channels per sample:
 
-        - Time (pandas.Timedelta): session timestamp (time only)
+        - Time (datetime.timedelta): session timestamp (time only)
         - Status (str): contains track status changes as numeric values (described below)
         - Message (str): contains the same information as status but in easily understandable
           words ('Yellow', 'AllClear',...)
@@ -934,10 +978,14 @@ def track_status_data(path, response=None, livedata=None):
     data = {'Time': [], 'Status': [], 'Message': []}
 
     for entry in response:
+        if len(entry) < 2:
+            continue
         row = entry[1]
+        if not isinstance(row, dict):
+            continue
         data['Time'].append(to_timedelta(entry[0]))
-        data['Status'].append(row['Status'])
-        data['Message'].append(row['Message'])
+        data['Status'].append(row.get('Status', ''))
+        data['Message'].append(row.get('Message', ''))
 
     return data
 
@@ -949,7 +997,7 @@ def session_status_data(path, response=None, livedata=None):
     Session status contains information on when a session was started and when it ended (amongst others). It
     provides the following data channels per sample:
 
-        - Time (pandas.Timedelta): session timestamp (time only)
+        - Time (datetime.timedelta): session timestamp (time only)
         - Status (str): status messages
 
     A new value is sent every time the session status changes.
@@ -977,8 +1025,14 @@ def session_status_data(path, response=None, livedata=None):
     data = {'Time': [], 'Status': []}
 
     for entry in response:
+        if len(entry) < 2:
+            continue
+        row = entry[1]
+        if not isinstance(row, dict) or 'Status' not in row:
+            continue
+
         data['Time'].append(to_timedelta(entry[0]))
-        data['Status'].append(entry[1]['Status'])
+        data['Status'].append(row['Status'])
 
     return data
 
@@ -1001,17 +1055,33 @@ def fetch_page(path, name):
     page = pages[name]
     is_stream = 'jsonStream' in page
     is_z = '.z.' in page
-    r = requests.get(base_url + path + pages[name], headers=headers)
+    if requests_session is None:
+        session = requests.session()
+    else:
+        session = requests_session
+    r = session.get(base_url + path + pages[name], headers=headers)
     if r.status_code == 200:
         raw = r.content.decode('utf-8-sig')
         if is_stream:
             records = raw.split('\r\n')[:-1]  # last split is empty
-            if name == 'position':
-                # Special case to improve memory efficency
+            if name in ('position', 'car_data'):
+                # Special case to improve memory efficiency
                 return records
             else:
+                decode_error_count = 0
                 tl = 12  # length of timestamp: len('00:00:00:000')
-                return [[e[:tl], parse(e[tl:], zipped=is_z)] for e in records]
+                ret = list()
+                for e in records:
+                    try:
+                        ret.append([e[:tl], parse(e[tl:], zipped=is_z)])
+                    except json.JSONDecodeError:
+                        decode_error_count += 1
+                        continue
+                if decode_error_count > 0:
+                    logging.warning(f"Failed to decode {decode_error_count}"
+                                    f" messages ({len(records)} messages "
+                                    f"total)")
+                return ret
         else:
             return parse(raw, is_z)
     else:
