@@ -53,16 +53,6 @@ Functions
     get_session
     get_round
 
-
-Exceptions
-----------
-
-:class:`NoLapDataError` can be raised when calling :meth:`Session.load_laps`
-if the API does not return any usable data.
-
-
-API Reference
--------------
 """
 
 from fastf1 import ergast
@@ -89,11 +79,11 @@ TESTING_LOOKUP = {'2020': [['2020-02-19', '2020-02-20', '2020-02-21'],
 D_LOOKUP = [[44, 'HAM', 'Mercedes'], [77, 'BOT', 'Mercedes'],
             [55, 'SAI', 'Ferrari'], [16, 'LEC', 'Ferrari'],
             [33, 'VER', 'Red Bull'], [11, 'PER', 'Red Bull'],
-            [2, 'RIC', 'McLaren'], [4, 'NOR', 'McLaren'],
+            [3, 'RIC', 'McLaren'], [4, 'NOR', 'McLaren'],
             [5, 'VET', 'Aston Martin'], [18, 'STR', 'Aston Martin'],
-            [3, 'RIC', 'Alpine'], [31, 'OCO', 'Alpine'],
+            [14, 'ALO', 'Alpine'], [31, 'OCO', 'Alpine'],
             [22, 'TSU', 'AlphaTauri'], [10, 'GAS', 'AlphaTauri'],
-            [47, 'SCH', 'Haas F1 Team'], [9, 'MAZ', 'Haas F1 Team'],
+            [47, 'MSC', 'Haas F1 Team'], [9, 'MAZ', 'Haas F1 Team'],
             [7, 'RAI', 'Alfa Romeo'], [99, 'GIO', 'Alfa Romeo'],
             [6, 'LAT', 'Williams'], [63, 'RUS', 'Williams']]
 
@@ -105,6 +95,21 @@ def get_session(year, gp, event=None):
     corresponding to the requested session.
     If no session is specified, the full weekend is returned.
 
+    Examples:
+
+        Get the second free practice of the first race of 2021::
+
+            get_session(2021, 1, 'FP2')
+
+        Get the qualifying of the 2020 Austrian Grand Prix::
+
+            get_session(2020, 'Austria', 'Q')
+
+        Get the second day of pre-season testing of 2021::
+
+            get_session(2021, 'testing', 2)
+
+
     Args:
         year (number): Session year
         gp (number or string): Name or weekend number (1: Australia,
@@ -556,7 +561,7 @@ class Telemetry(pd.DataFrame):
             resampled_columns['Source'] = res_source
 
             # join resampled columns and make 'Date' a column again
-            merged = Telemetry(resampled_columns).reset_index().rename(columns={'index': 'Date'})
+            merged = Telemetry(resampled_columns, session=self.session).reset_index().rename(columns={'index': 'Date'})
 
             # recalculate the time columns
             merged['SessionTime'] = merged['Date'] - self.session.t0_date
@@ -628,8 +633,10 @@ class Telemetry(pd.DataFrame):
             if ch not in self.columns:
                 continue
             sig_type = self._CHANNELS[ch]['type']
-
             if sig_type == 'continuous':  # yes, this is necessary to prevent pandas from crashing
+                if ret[ch].dtype == 'object':
+                    warnings.warn("Interpolation not possible for telemetry "
+                                  "channel because dtype is 'object'")
                 missing = self._CHANNELS[ch]['missing']
                 ret.loc[:, ch] = ret.loc[:, ch]\
                     .interpolate(method=missing, limit_direction='both', fill_value='extrapolate')
@@ -1112,8 +1119,9 @@ class Session:
             data of different laps.
 
         Args:
-            with_telemetry (bool): Load telemetry data also. (Same as calling :meth:`Session.load_telemetry` manually)
-            livedata (fastf1.api.LivetimingResponse, optional) :
+            with_telemetry (bool): Load telemetry data also.
+                (Same as calling :meth:`Session.load_telemetry` manually)
+            livedata (:class:`fastf1.livetiming.data.LiveTimingData`, optional) :
                 instead of requesting the data from the api, locally saved
                 livetiming data can be used as a data source
 
@@ -1303,7 +1311,7 @@ class Session:
         :attr:`Session.laps`.
 
         Args:
-            response (fastf1.api.LivetimingResponse, optional) :
+            livedata (:class:`fastf1.livetiming.data.LiveTimingData`, optional) :
                 instead of requesting the data from the api, locally saved
                 livetiming data can be used as a data source
         """
